/*
 * Jalview - A Sequence Alignment Editor and Viewer
 * Copyright (C) 2005 AM Waterhouse, J Procter, G Barton, M Clamp, S Searle
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 */

package jalview.appletgui;

import java.util.*;

import java.awt.event.*;

import java.awt.*;

import jalview.datamodel.*;
import jalview.appletgui.PaintRefresher;

public class RedundancyPanel extends SliderPanel implements Runnable, WindowListener
{
  AlignmentPanel ap;

  Stack historyList = new Stack(); // simpler than synching with alignFrame.
  Vector originalSequences;
  Hashtable originalColours;
  SequenceI[] oldAlignment;
  float [] redundancy;
  Frame frame;

  public RedundancyPanel(AlignmentPanel ap)
  {
    super(ap, 0, false, null);

    this.ap = ap;
    undoButton.setVisible(true);
    applyButton.setVisible(true);
    allGroupsCheck.setVisible(false);

    label.setText("Enter the redundancy threshold");
    valueField.setText("100");

    slider.setVisibleAmount(1);
    slider.setMinimum(40);
    slider.setMaximum(100+slider.getVisibleAmount());
    slider.setValue(100);

    slider.addAdjustmentListener(new AdjustmentListener()
    {
      public void adjustmentValueChanged(AdjustmentEvent evt)
      {
        valueField.setText(slider.getValue() + "");
        sliderValueChanged();
      }
    });

    frame = new Frame();
    frame.add(this);
    jalview.bin.JalviewLite.addFrame(frame, "Redundancy threshold selection",
                                     400, 100);

    frame.addWindowListener(this);

    Thread worker = new Thread(this);
    worker.start();
  }
  /**
   * This is a copy of remove redundancy in jalivew.datamodel.Alignment
   * except we dont want to remove redundancy, just calculate once
   * so we can use the slider to dynamically hide redundant sequences
   *
   * @param threshold DOCUMENT ME!
   * @param sel DOCUMENT ME!
   *
   * @return DOCUMENT ME!
   */
  public void run()
  {
      label.setText("Calculating....");

      slider.setVisible(false);
      applyButton.setEnabled(false);
      valueField.setVisible(false);

      validate();

      Vector sel = new Vector();
      SequenceGroup sg = ap.av.getSelectionGroup();
      int height;
      originalSequences = new Vector();
      originalColours = new Hashtable();

      if ( (sg != null) && (sg.getSize(false) >= 1))
      {
         height = sg.getSize(false);
        for (int i = 0; i < sg.getSize(false); i++)
        {
          sel.addElement(sg.getSequenceAt(i));
        }
      }
      else
      {
        height = ap.av.alignment.getHeight();
        for (int i = 0; i < ap.av.alignment.getHeight(); i++)
        {
          sel.addElement(ap.av.alignment.getSequenceAt(i));
        }
      }

      redundancy = new float[height];
      for (int i = 0; i < height; i++)
      {
        redundancy[i] = 0f;
      }


     // long start = System.currentTimeMillis();

      float pid;
      SequenceI seqi, seqj;
      for (int i = 0; i < sel.size(); i++)
      {
          originalSequences.addElement(sel.elementAt(i));
          originalColours.put(sel.elementAt(i),
                             ((SequenceI) sel.elementAt(i)).getColor());

          for (int j = 0; j < i; j++)
          {
            if(i==j)
              continue;

            seqi = (SequenceI) sel.elementAt(i);
            seqj = (SequenceI) sel.elementAt(j);

            if (sg != null)
            {
              pid = jalview.util.Comparison.PID(seqi,
                                                seqj,
                                                sg.getStartRes(), sg.getEndRes());
            }
            else
              pid = jalview.util.Comparison.PID( seqi,  seqj );


            if(seqj.getLength() < seqi.getLength())
              redundancy[j] = Math.max(pid, redundancy[j]);
            else
              redundancy[i] = Math.max(pid, redundancy[i]);

          }
      }


      label.setText("Enter the redundancy threshold");
      slider.setVisible(true);
      applyButton.setEnabled(true);
      valueField.setVisible(true);

      validate();
     // System.out.println("blob done "+ (System.currentTimeMillis()-start));
  }

  void sliderValueChanged()
  {
    if(redundancy==null)
      return;

    float value = slider.getValue();

    for(int i=0; i<redundancy.length; i++)
    {
      if (value > redundancy[i])
         ((SequenceI)originalSequences.elementAt(i)).setColor(java.awt.Color.white);
      else
         ((SequenceI)originalSequences.elementAt(i)).setColor(java.awt.Color.red);
    }

    PaintRefresher.Refresh(null,ap.av.alignment);

    }
  public void applyButton_actionPerformed()
  {
    historyList.push(new HistoryItem("Remove redundancy",
                        ap.av.alignment, HistoryItem.HIDE));

            if ((historyList.size() == 1) ||
                    !ap.alignFrame.historyList.contains(historyList.firstElement()))
            {
                ap.alignFrame.addHistoryItem((HistoryItem) historyList.firstElement());
                ap.alignFrame.updateEditMenuBar();
            }

            Vector del = new Vector();

            undoButton.setEnabled(true);

            float value = slider.getValue();
            SequenceGroup sg = ap.av.getSelectionGroup();

            for (int i = 0; i < redundancy.length; i++)
            {
              if (value <= redundancy[i])
              {
                SequenceI seq = (SequenceI) originalSequences.elementAt(i);
                ap.av.alignment.deleteSequence(seq);
                del.addElement(seq);
                if (sg != null)
                {
                  sg.deleteSequence(seq, false);
                }
              }
            }


            // This has to be done before the restoreHistoryItem method of alignFrame will
            // actually restore these sequences.
            if (del.size() > 0)
            {
                for (int i = 0, j = del.size(); i < j; i++)
                {
                    SequenceI sq = (SequenceI) del.elementAt(i);
                    sq.deleteChars(0, sq.getLength());
                }
            }

        ap.av.firePropertyChange("alignment", null, ap.av.getAlignment().getSequences());
        ap.alignFrame.updateEditMenuBar();

  }

  public void undoButton_actionPerformed()
  {
    HistoryItem hi = (HistoryItem) historyList.pop();
    ap.alignFrame.restoreHistoryItem(hi);

    if (historyList.size() == 0)
    {
      undoButton.setEnabled(false);
    }
    ap.alignFrame.updateEditMenuBar();
  }

  public void valueField_actionPerformed(ActionEvent e)
  {
    try
    {
      int i = Integer.parseInt(valueField.getText());
      slider.setValue(i);
    }
    catch (Exception ex)
    {
      valueField.setText(slider.getValue() + "");
    }
  }


  public void windowOpened(WindowEvent evt)
  {}

  public void windowClosing(WindowEvent evt)
  {
    for(int i=0; i<originalSequences.size(); i++)
    {
      SequenceI seq = (SequenceI)originalSequences.elementAt(i);
      seq.setColor( (java.awt.Color)originalColours.get(seq));
    }

    PaintRefresher.Refresh(ap.av.alignment);
  }

  public void windowClosed(WindowEvent evt)
  {}

  public void windowActivated(WindowEvent evt)
  {}
  public void windowDeactivated(WindowEvent evt)
  {}
  public void windowIconified(WindowEvent evt)
  {}
  public void windowDeiconified(WindowEvent evt)
  {}
}
